#ifndef CONTROL_H
#define CONTROL_H


#ifdef WIN32
#pragma warning(disable:4786)
#endif


#include <string>
#include <map>
#include <iostream>
#include <cfloat>
#include "Utilities.h"

using namespace std;

#include "Timer.h"
#include "Random.h"
#include <map>

class Control {
private:
  Timer timer;
  
  int    nrTry, maxTry;
  double bestCost;

  unsigned long partial_Iter;
  int    seed;
  double timeLimit;
  ostream *os;
  ostream *oss;
  const char *ss;
  
public:

  Control(Random *rnd, int, char** ); // construct a control object, given a set of command line options
  ~Control();
  const char *is;
  int init;  
  int test_version;
  int Neighborhood;
  unsigned int solver;
  unsigned long max_iter;
  bool print_statistics;
 
  Color k;
  Color finalK;
  Color best_k;

  void Usage();
  void ComParser(int argc, char** argv );
  int getSeed() { return seed; } // get the seed for the random number generator
  void resetTime(); // reset the timer
  double getTime(); // get the timer's time
  bool timeLeft() { return ( getTime() < timeLimit ); }; // check whether our run has time left
  int getNumTry() { return (nrTry); };  

  void beginTry(); // begin a run
  void endTry(); // end a run
  
  void setCurrentCost(fvalue cost, Color cl, unsigned long iter); // tell control the current cost of your solution
  void setPartialIter(unsigned long iter) {partial_Iter = iter;}; 
  void IncrPartialIter(unsigned long iter) {partial_Iter += iter;}; 
    
  const char *getInputStream() { return is; } // get the input stream for constructing problems
  const char *getInputSolutionStream() { return ss; } // get the input stream for constructing problems
  ostream& getOutputStream() { return *os; } // get the output stream used for writing output data
  ostream& getOutputSolutionStream() { return *oss; } // get the input stream for constructing problems
  int getNumberOfTries() { return maxTry; } // get the max number of tries
  bool triesLeft() { return ( nrTry < maxTry ); } // check whether more tries need to be run
  double getTimeLimit() { return timeLimit; } // get the time limit of one run
  void informationsReport();
  char *readline (istream &infile);
  char *match (char *source, char *prefix);
  void SetDefaultParameters();
  Random *rg;

};

#endif
