#include "Input.h"
#include "Solution.h"
#include <string>


// Input Methods

Graph::Graph()
 {
   num_of_vertices = 0;
   num_of_edges = 0;
   adjacency_matrix = NULL;
 }


void Graph::Load(const char *id, bool print_statistics) 
  {
    char *tmp;     // temporary std::std::string
    //tmp = strtok(strdup(id.c_str()), ":");
    char buf[255]; // line buffer
    //int nn, en = 0;
    int cen = 0; // for doublechecking file definition (counts the read edges)
    ifstream is(id);
    
    if (!is.is_open())
      {
	cerr<<"Instance file not found\n";
	exit(1);
      }

    while (!is.eof()) { 
      is.getline(buf, 255); // process each line
      //printf("%s\n",buf);fflush(stdout);
      switch (buf[0]) {
      case 'p':
	{
	  char     *p=(char*)(&buf);//strstr((char*)(&buf), "p");
	  if(strstr((char*)&buf,"edge")!=NULL) {
	    p++;
	    while(*p != '\0' && *p != '\n'){
	      while(*p == ' ')
		p++;
	      while(*p != ' ')
		p++;
	      while(*p == ' ')
		p++;
	      num_of_vertices = atoi(p);
	      while(*p != ' ')
		p++;
	      while(*p == ' ')
		p++;
	      *p = '\0';
	    }
	  }
	  adjacency_matrix = new unsigned char *[num_of_vertices];
	  for (Vertex v = 0; v < num_of_vertices; v++) {
	    adjacency_matrix[v] = new unsigned char[num_of_vertices];
	    for (Vertex w = 0; w < num_of_vertices; w++)
	      adjacency_matrix[v][w] = 0;
	  }
	  degree.resize(num_of_vertices,0);
	  adjacency_set.resize(num_of_vertices);
	}
	break;
      case 'e':
	{
	  // reads the edge information and add the edge between the 
	  // corresponding nodes
	  tmp = strtok(buf, " "); // splits buf among the spaces
	  tmp = strtok(0, " "); // consider the first number
	  Vertex v = atoi(tmp) - 1;
	  tmp = strtok(0, " "); // and now the second one
	  Vertex w = atoi(tmp) - 1;
	  if (adjacency_matrix[v][w] == 0)
	    cen++;
	  adjacency_matrix[v][w] = adjacency_matrix[w][v] = 1;
	}
	break;
      case 'c':
      case ' ':
      case '\n':
	// ignore the comments and the empty lines
	break;
      default:
	//std::cerr << "Error while reading the input file " << id << std::endl
	//  << "the line " << buf << " was not understandable" << std::endl;
	break;
      }
    }
    num_of_edges = cen;
    
    //updating vector degree
     av_degree = 0;

     max_degree = 0;
     min_degree = INT_MAX;
    for (Vertex i=0;i<num_of_vertices;i++)
      {
	adjacency_set[i].clear();
	degree[i]=0;
	for (Vertex j=0;j<num_of_vertices;j++)
	  {
	    if (Edge(i,j))
	      {
		degree[i]++;
		adjacency_set[i].insert(j);
	      }
	  }
	if (degree[i]>max_degree)
	  max_degree = degree[i];
	if (degree[i]<min_degree)
	  min_degree=degree[i];
	
	av_degree += degree[i];
      }
    
    av_degree /= num_of_vertices;
    density = ((float)(num_of_edges))/(((float) (num_of_vertices)/2)*(num_of_vertices-1));  

    if (print_statistics)
      {
	for (Vertex i = 0; i < num_of_vertices; i++)
	{
	  //cout<<(double) degree[i]/(num_of_vertices-1)<<" ";
	}
	//cout<<endl;
	double av_degree_norm = av_degree/(num_of_vertices-1);
	double std_degree;
	double sum_sd = 0;
	for (Vertex i=0;i<num_of_vertices;i++)
	  {
	    sum_sd += (degree[i] - av_degree) * (degree[i] - av_degree);
	  }
	std_degree = sqrt(sum_sd/num_of_vertices);
	
	printf("%d %.4f %.1f %.1f %.4f %.4f %d %d \n",
	       num_of_vertices,density,av_degree,std_degree,
	       av_degree_norm,(std_degree/(num_of_vertices-1)),
	       min_degree, max_degree);
      }
}


std::ostream& operator<<(std::ostream& os, const Graph& g) 
  {
    os << "graph ";
    for (Vertex v = 0; v < g.Vertices(); v++)
      for (Vertex w = v + 1; w < g.Vertices(); w++) 
	if (g.Edge(v,w))
	  os << "[" << w << "," << v << "] ";
    return os;
  }

Graph::~Graph()
  {
    if (adjacency_matrix != NULL)
      {
	for (Vertex v = 0; v < num_of_vertices; v++)
	  delete [] adjacency_matrix[v];
	delete [] adjacency_matrix;
      }
  }

