#include <algorithm>
#include <cmath>

#include <iostream>
#include <fstream>
#include "SolutionManager.h"


SolutionManager::SolutionManager(Random *rnd, Graph* pin = NULL, Control *ctrl = NULL)
{
  p_in = pin;
  rg = rnd;
  p_ctrl=ctrl;
}

 
fvalue SolutionManager::computeFitness(Coloring &ass)
  {
    ass.fitness = 0;
    for (Vertex v = 0; v < p_in->Vertices(); v++)
    {
      for (Vertex w = v+1; w < p_in->Vertices(); w++)
	{
	  if (p_in->Edge(v,w) && (ass.col[v] == ass.col[w])) 
	    {
	      ass.fitness++;
	    }
	}
    }
    return ass.fitness;
  }



 
void SolutionManager::updateColoringData(Coloring &ass) const 
{
  const Graph &g = *p_in; // g is an alias for the input Graph
  
  ass.fitness = 0;

  for (Vertex i=0; i<g.Vertices(); i++)
  {
    for (Vertex j=i; j<g.Vertices(); j++)
	{
	   if (g.Edge(i,j) && (ass.col[i] == ass.col[j])) 
	    {
	      ass.fitness++;
	    }
	} 
    }
  if (ass.fitness != 0)
    ass.feasible = false;
  else
    ass.feasible = true;
}



void SolutionManager::randomAssignment(Coloring& ass, Color num_col)
{
    Color rnd;
    ass.SetColors(num_col);
    for (Vertex v = 0; v < p_in->Vertices(); v++)
    {
	rnd = (unsigned int) rg->ran0N(num_col-1);
	ass.col[v]=rnd;
    }
    updateColoringData(ass);
}





std::ostream& operator<<(std::ostream& os, const Move& mv)
{
  os <<"Cost: "<<mv.delta;
  return os;
}


Move::Move()
{
  delta = 0;
}

Move::Move(const Move& a)
{
  delta = a.delta;
}

Move& Move::operator=(const Move& a)
{
  delta = a.delta;
  return *this;
}


bool Move::operator==(const Move &a)
{ 
  if (delta != a.delta)
    return false;
  return true;
}
