#include <iostream>
#include <sstream>
#include <stdexcept>
#include <string>
#include "Control.h"

using namespace std;
using std::ostringstream; using std::string;



%%

i file "f inputfile (DIMACS format necessary)"	reqarg
o out	"f outputfile "				reqarg
n num	" number of tests"				reqarg
t time	" maximum time"				reqarg
m max	" number of iterations for each trial (optarg)"	optarg
s seed	" seed"					reqarg
  init " way of initializing a solution: "		reqarg
  neigh  " the neighborhood"				reqarg
  solver " Solver: "                                  reqarg
  test " run test version (1) or not (0) (optarg)"		optarg
  stat " prints instance statistics and reduction (optarg)"	optarg
k initK   " initial coloring"							reqarg	
K finalK   " final coloring"							reqarg	
h help  "prints help information"					return
  morehelp  "prints some more help information"					return
  
%%

void Control::ComParser(int argc, char** argv ) 
  {
    struct options OPT;
    
    const char *progname;

  progname = argv [0] != 0 && *(argv [0]) != '\0'
    ? argv [0]
    : "gcp";

  try
    {
      int i = parse_options (&OPT, argc, argv);

      if (OPT.opt_help)
	{
	  //Usage();
	   std::cout << "Usage: " << progname
                             << " [OPTION]... [ARGUMENT]...\n\n"
                             << "Options:\n" STR_HELP;
	  exit(1);
	}
	  
       if(OPT.opt_morehelp)
	{	
	  Usage();
	  exit(1);
	}

      if(i <= 1)
	{
	  //Usage();
	   std::cout << "Usage: " << progname
                             << " [OPTION]... [ARGUMENT]...\n\n"
                             << "Options:\n" STR_HELP;
	   exit(1);
	}

      if (OPT.opt_file)
	{
	  is = OPT.arg_file;
	}
      else
	{
	  cerr<<"Instance file missing!\n"<<endl;
	  exit(1);
	}

      if(OPT.opt_out)
	{
	  os = new ofstream(OPT.arg_out);
	  string file_solution(OPT.arg_out);
	  file_solution.append(".sol");
	  oss = new ofstream(file_solution.c_str());
	} else {
	  cerr << "Warning: No output file given, writing to stdout" << endl;
	  os = &cout;
	  oss = &cout;
      } 
      
      if(OPT.opt_num)
	{
	  maxTry = atoi(OPT.arg_num);
	} else {
	  cerr << "Warning: Number of tries is set to default (1)" << endl;
	}

      if(OPT.opt_time)
	{
	  timeLimit = atof(OPT.arg_time);
	} else {
	  cerr << "Warning: Time limit is set to default DBL_MAX" << endl;
	}

      if(OPT.opt_max)
	{
	  if (OPT.arg_max != 0)
	    max_iter = atoi(OPT.arg_max);
	  else
	    {
	      max_iter = 0;
	      cerr << "Warning: Max iter is set to 10000 |V|"<<endl;
	    }
	} else {
	  cerr << "Warning: Max iter is set to very large number (LONG_MAX)"<<endl;
	}

       if(OPT.opt_seed)
	{
	  seed = atoi(OPT.arg_seed);
	  rg->set_seed( seed );
	} else {
	  cerr << "Warning: " << seed << " used as default random seed" << endl;
	}
	
      if(OPT.opt_init) {
	init = atoi(OPT.arg_init);
      } else {
	cerr << "Warning: init not set" << endl;
      }

      if(OPT.opt_neigh)
	{
	  Neighborhood = atoi(OPT.arg_neigh);
	} else {
	  cerr << "Warning: neigh not set" << endl;
	}

      if(OPT.opt_solver)
	{
	  solver = atoi(OPT.arg_solver);
	} else {
	  cerr << "Warning: solver not set" << endl;
	}

      if (OPT.opt_test)
	{
	  test_version = 1;
	  cout<<"Running Test Version"<<endl;
	} 

      if(OPT.opt_stat)
	{
	  print_statistics = true;
	}

      if (OPT.opt_initk)
	{  
	  k = atoi(OPT.arg_initk);
	} else {
	  cerr << "Warning: No initial color set" << endl;
	}

      if (OPT.opt_finalk)
	{  
	  finalK = atoi(OPT.arg_finalk);
	} else {
	  cerr << "Warning: No initial color set" << endl;
	}
	
  }
  catch (std::runtime_error e)
    {
      std::cerr << progname << ": " << e.what () << "\n"
	"Try `" << progname
		<< " --help' for more information.\n";
      exit(1);
    }
  }
